#pragma once

#include <type_traits>

#include "future.pre.h"
#include "../misc.h"
#include "../result.h"

namespace asyncpp
{

    template<
        typename T_value,
        typename T_derived>
    struct base_future
        : public tag_future
    {
    public:
        using value_type    = T_value;
        using result_type   = future_result<value_type>;
        using derived_type  = T_derived;
        using this_type     = base_future<value_type, derived_type>;

    public:
        /**
         * @brief Transform the result of this future.
         */
        template<
            chaining_mode X_mode = copy,
            typename X_lambda>
        inline auto map(X_lambda&& p_lambda) &;

        /**
         * @brief Transform the result of this future.
         */
        template<
            chaining_mode X_mode = move,
            typename X_lambda>
        inline auto map(X_lambda&& p_lambda) &&;

    public:
        /**
         * @brief Execute the given lambda after the future is finished and
         *        wait for the future returned by the lambda.
         */
        template<
            chaining_mode X_mode = copy,
            typename X_lambda>
        inline auto and_then(X_lambda&& p_lambda) &;

        /**
         * @brief Execute the given lambda after the future is finished and
         *        wait for the future returned by the lambda.
         */
        template<
            chaining_mode X_mode = move,
            typename X_lambda>
        inline auto and_then(X_lambda&& p_lambda) &&;

    #ifdef ASYNCPP_ENABLE_FEATURE_TIMING
    public:
        /**
         * @brief Throw an execption if the timeout has passed.
         *
         * This method is only enabled if timer.h is included before.
         */
        template<
            chaining_mode X_mode = copy,
            typename X_base,
            typename X_ratio>
        inline auto timeout(const duration<X_base, X_ratio>& p_timeout) &;

        /**
         * @brief Throw an execption if the timeout has passed.
         *
         * This method is only enabled if timer.h is included before.
         */
        template<
            chaining_mode X_mode = move,
            typename X_base,
            typename X_ratio>
        inline auto timeout(const duration<X_base, X_ratio>& p_timeout) &&;
    #endif
    };

    /**
     * @brief Create a lazy evaluated future.
     */
    template<typename X_lambda>
    inline auto lazy(X_lambda&& p_lambda);

}
