#pragma once

#include "future.h"

namespace asyncpp {
namespace __future {

    template<typename T_lambda>
    struct lazy_future final :
        public base_future<
            decltype(std::declval<T_lambda>()()),
            lazy_future<T_lambda>
        >
    {
    public:
        using lambda_type       = T_lambda;
        using value_type        = decltype(std::declval<lambda_type>()());
        using this_type         = lazy_future<lambda_type>;
        using base_future_type  = base_future<value_type, this_type>;
        using result_type       = typename base_future_type::result_type;

    private:
        lambda_type _lambda;

    public:
        /**
         * @brief Constructor.
         */
        template<typename X_lambda>
        inline lazy_future(
            X_lambda&& p_lambda)
            : _lambda(std::forward<X_lambda>(p_lambda))
            { }

        inline lazy_future(lazy_future &&) = default;
        inline lazy_future(lazy_future const &) = default;

    public: /* future */
        /**
         * @brief Poll the result from the future.
         */
        template<typename X = value_type>
        inline auto poll()
            -> std::enable_if_t<
                std::is_void_v<X>,
                result_type>
        {
            _lambda();
            return result_type::ready();
        }

        /**
         * @brief Poll the result from the future.
         */
        template<typename X = value_type>
        inline auto poll()
            -> std::enable_if_t<
                !std::is_void_v<X>,
                result_type>
        {
            return result_type::ready(_lambda());
        }
    };

} }

namespace asyncpp
{

    template<typename X_lambda>
    inline auto lazy(X_lambda&& p_lambda)
    {
        using lambda_type = X_lambda;
        using lazy_type   = __future::lazy_future<lambda_type>;

        return lazy_type(std::forward<X_lambda>(p_lambda));
    }

}
