#pragma once

namespace asyncpp
{

    enum chaining_mode
    {
        move,
        copy,
        ref,
    };

    /**
     * @brief Get the storage type of the passed type for the passed mode.
     *
     * move: T
     * copy: T
     * ref   T &
     */
    template<chaining_mode X_mode, typename X_type, typename = void>
    struct storage_type;

    /**
     * @brief Get the forward type of the passed type for the passed mode.
     *
     * move: T &&
     * copy: T const &
     * ref   T &
     */
    template<chaining_mode X_mode, typename X_type, typename = void>
    struct forward_type;

    template<chaining_mode X_mode, typename X_type>
    using storage_type_t = typename storage_type<X_mode, X_type>::type;

    template<chaining_mode X_mode, typename X_type>
    using forward_type_t = typename forward_type<X_mode, X_type>::type;

}
