#pragma once

#include <asyncpp/timing/timeout/timeout_stream.h>

#include "stream.h"

namespace asyncpp {
namespace __stream {

     template<
        chaining_mode X_mode,
        typename X_self,
        typename X_base,
        typename X_ratio>
    auto timeout_impl(
        X_self&&                            p_self,
        const duration<X_base, X_ratio>&    p_timeout)
    {
        using self_type             = X_self;
        using derived_type          = typename std::decay_t<self_type>::derived_type;
        using derived_storage_type  = storage_type_t<X_mode, derived_type>;
        using derived_ref_type      = std::conditional_t<
                                        std::is_const_v<std::remove_reference_t<self_type>>,
                                        derived_type const &,
                                        derived_type &>;
        using derived_forward_type  = forward_type_t<X_mode, derived_ref_type>;
        using timeout_stream_type   = timing::timeout_stream<derived_storage_type>;

        auto& self = static_cast<derived_ref_type>(p_self);

        return timeout_stream_type(
            std::forward<derived_forward_type>(self),
            p_timeout);
    }

} }

namespace asyncpp
{

    template<
        typename T_value,
        typename T_derived>
    template<
        chaining_mode X_mode,
        typename X_base,
        typename X_ratio>
    auto base_stream<T_value, T_derived>
        ::timeout(const duration<X_base, X_ratio>& p_timeout) &
    {
        return __stream::timeout_impl<X_mode>(std::move(*this), p_timeout);
    }

    template<
        typename T_value,
        typename T_derived>
    template<
        chaining_mode X_mode,
        typename X_base,
        typename X_ratio>
    auto base_stream<T_value, T_derived>
        ::timeout(const duration<X_base, X_ratio>& p_timeout) &&
    {
        static_assert(
            X_mode != ref,
            "Can not store rvalue reference as lvalue reference!");

        return __stream::timeout_impl<X_mode>(std::move(*this), p_timeout);
    }

}
