#pragma once

#include <memory>

namespace asyncpp
{

    struct task
    {
    public:
        struct handle
        {
        public:
            /**
             * @brief Destructor.
             */
            virtual ~handle() = default;

            /**
             * @brief Notify that a resource, the task is waiting for is ready.
             */
            virtual void notify() = 0;
        };

        using handle_ptr_s = std::shared_ptr<handle>;
        using handle_ptr_w = std::weak_ptr<handle>;

    private:
        handle_ptr_w _handle;

    public:
        /**
         * @brief Default constructor.
         */
        inline task();

        /**
         * @brief Constructor.
         */
        inline task(const handle_ptr_w& p_handle);

        /**
         * @brief Notify that a resource, the task is waiting for is ready.
         */
        inline void notify() const;

    public:
        /**
         * @brief Get the current task.
         */
        static inline task current();

    public:
        struct lock
        {
        public:
            /**
             * @brief Constructor.
             */
            inline lock(task::handle_ptr_s p_handle);

            inline lock(lock &&) = delete;
            inline lock(lock const &) = delete;

            /**
             * @brief Destructor.
             */
            inline ~lock();
        };

    private:
        struct storage
        {
            handle_ptr_w current;
        };

        /**
         * @brief Get the thread local storage.
         */
        static storage& local_storage();
    };

}
