#pragma once

#include <cppcore/misc/exception.h>

#include "task.h"

namespace asyncpp
{

    /* task::lock */

    task::lock::lock(task::handle_ptr_s p_handle)
    {
        auto& current = local_storage().current;
        if (!current.expired())
            throw cppcore::invalid_operation_exception("Current task is already assigned!");
        current = p_handle;
    }

    task::lock::~lock()
        { local_storage().current.reset(); }

    /* task*/

    task::task()
        : _handle()
        { }

    task::task(const handle_ptr_w& p_handle)
        : _handle(p_handle)
        { }

    void task::notify() const
    {
        auto s = _handle.lock();
        if (s)
            s->notify();
    }

    task task::current()
        { return task(local_storage().current); }

    task::storage& task::local_storage()
    {
        thread_local storage value;
        return value;
    }

}
