#pragma once

#include <queue>

#include "executor.h"

namespace asyncpp {
namespace executor {

    template<typename T_runtime>
    struct current_thread
        : public executor
    {
    public:
        using runtime_type      = T_runtime;
        using task_map_type     = std::map<task_handle*, task_handle_ptr_s>;
        using task_queue_type   = std::queue<task_handle_ptr_s>;

    private:
        runtime_type    _runtime;           //!> Runtime to call when executor is idling
        task_queue_type _pending_tasks;     //!< Tasks ready for execution.
        task_map_type   _sleeping_tasks;    //!< Tasks waiting for resource.

    public:
        /**
         * @brief Constructor.
         */
        template<typename X_runtime>
        inline current_thread(X_runtime&& p_runtime);

        /**
         * @brief Run the executor.
         *
         * It will return if all tasks are finished.
         */
        void run();

        /**
         * @brief Spawn the passed future and run the executor.
         *
         * It will return if all tasks are finished.
         */
        template<typename X_future>
        void run(X_future&& p_future);

    private:
        /**
         * @brief Run the executor.
         *
         * It will return if all tasks are finished.
         */
        void run_impl();

        /**
         * @brief Poll the passed task.
         */
        void poll_task(task_handle_ptr_s p_handle);

        /**
         * @brief Callback to inform the executor about notified tasks.
         */
        void task_notified(task_handle& p_handle) override;

    private: /* executor */
        /**
         * @brief Actual implementation of the spawn method.
         */
        void spawn_impl(task_handle_ptr_s p_handle) override;
    };

} }

#include "current_thread.inl"
