#pragma once

#include "current_thread.h"

#include "executor.inl"

namespace asyncpp {
namespace executor {

    /* current_thread */

    template<typename T_runtime>
    template<typename X_runtime>
    current_thread<T_runtime>
        ::current_thread(X_runtime&& p_runtime)
            : _runtime(std::forward<X_runtime>(p_runtime))
            { }

    template<typename T_runtime>
    void current_thread<T_runtime>
        ::run()
    {
        auto executor_lock  = executor::lock(*this);
        auto runtime_lock   = _runtime.init_thread();

        (void)runtime_lock;

        run_impl();
    }

    template<typename T_runtime>
    template<typename X_future>
    void current_thread<T_runtime>
        ::run(X_future&& p_future)
    {
        auto executor_lock  = executor::lock(*this);
        auto runtime_lock   = _runtime.init_thread();

        (void)runtime_lock;

        spawn(std::forward<X_future>(p_future));

        run_impl();
    }

    template<typename T_runtime>
    void current_thread<T_runtime>
        ::run_impl()
    {
        while (true)
        {
            while (!_pending_tasks.empty())
            {
                auto t = _pending_tasks.front();
                _pending_tasks.pop();
                poll_task(t);
            }

            if (_sleeping_tasks.empty())
                return;

            _runtime.idle(nullptr);
        }
    }

    template<typename T_runtime>
    void current_thread<T_runtime>
        ::poll_task(task_handle_ptr_s p_handle)
    {
        task::lock l(p_handle);

        // TODO execption handling

        if (!p_handle->poll())
        {
            if (p_handle->notified())
                _pending_tasks.emplace(p_handle);
            else
                _sleeping_tasks.emplace(p_handle.get(), p_handle);
        }
    }

    template<typename T_runtime>
    void current_thread<T_runtime>
        ::task_notified(task_handle& p_handle)
    {
        auto it = _sleeping_tasks.find(&p_handle);
        if (it != _sleeping_tasks.end())
        {
            _pending_tasks.emplace(it->second);
            _sleeping_tasks.erase(it);
        }
    }

    template<typename T_runtime>
    void current_thread<T_runtime>
        ::spawn_impl(task_handle_ptr_s p_task)
        { _pending_tasks.push(p_task); }

} }
