#pragma once

#include "task_handle.h"

namespace asyncpp {
namespace executor {

    struct executor
    {
    public:
        friend __impl::task_handle;

        using task_handle = __impl::task_handle;
        using task_handle_ptr_s = std::shared_ptr<__impl::task_handle>;

    public:
        /**
         * @brief Spawn a new task.
         */
        template<typename X_future>
        static inline void spawn(X_future&& p_future);

    private:
        /**
         * @brief A task was notified that a resource it is waiting for is ready.
         */
        virtual void task_notified(task_handle& p_handle) = 0;

        /**
         * @brief Actual implementation of the spawn method.
         */
        virtual void spawn_impl(task_handle_ptr_s p_handle) = 0;

    protected:
        struct lock
        {
        public:
            /**
             * @brief Constructor.
             */
            inline lock(executor& p_current);

            inline lock(lock &&) = delete;
            inline lock(lock const &) = delete;

            /**
             * @brief Destructor.
             */
            inline ~lock();
        };

    private:
        struct storage
        {
            executor * current;
        };

        /**
         * @brief Get the thread local storage.
         */
        static inline storage& local_storage();
    };

} }
