#pragma once

#include <cppcore/misc/exception.h>

#include "executor.h"

#include "task_handle.inl"

namespace asyncpp {
namespace executor {

    /* executor */

    template<typename X_future>
    void executor::spawn(X_future&& p_future)
    {
        using future_type       = X_future;
        using task_handle_type  = __impl::task_handle_tpl<future_type>;

        auto exec = local_storage().current;
        if (!exec)
            throw std::runtime_error("Thread local executor instance is not assigned!");

        exec->spawn_impl(std::make_shared<task_handle_type>(
            *exec,
            std::forward<X_future>(p_future)));
    }

    executor::storage& executor::local_storage()
    {
        thread_local storage value;
        return value;
    }

    /* lock */

    executor::lock::lock(executor& p_current)
    {
        auto& current = executor::local_storage().current;
        if (current)
            throw cppcore::invalid_operation_exception("Current executor is already assigned!");
        current = &p_current;
    }

    executor::lock::~lock()
        { executor::local_storage().current = nullptr; }

} }
