#pragma once

#include <asyncpp/core/task.h>

namespace asyncpp {
namespace executor {

    struct executor;

    namespace __impl
    {

        struct task_handle
            : public task::handle
        {
        private:
            executor&   _owner;
            bool        _notified;

        public:
            /**
             * @brief Constructor.
             */
            inline task_handle(executor& p_owner);

            inline task_handle(task_handle &) = delete;
            inline task_handle(task_handle const &) = delete;

            /**
             * @brief Notify that a resource, the task is waiting for is ready.
             */
            inline void notify() override;

            /**
             * @brief Check if the task was notified.
             */
            inline bool notified() const;

            /**
             * @brief Poll the future stored in the task.
             *
             * @return TRUE if the task is finished, FALSE otherwise.
             */
            inline bool poll();

        private:
            /**
             * @brief Actual poll implementation.
             */
            virtual bool poll_impl() = 0;
        };

        template<typename T_future>
        struct task_handle_tpl
            : public task_handle
        {
        public:
            using future_type = T_future;

        private:
            future_type _future;

        public:
            /**
             * @brief Constructor.
             */
            template<typename X_future>
            inline task_handle_tpl(
                executor&   p_owner,
                X_future&&  p_future);

        private:
            /**
             * @brief Actual poll implementation.
             */
            inline bool poll_impl() override;
        };

    }

} }
