#pragma once

#include "executor.h"
#include "task_handle.h"

namespace asyncpp {
namespace executor {
namespace __impl {

    /* task_handle */

    task_handle::task_handle(
        executor& p_owner)
        : _owner    (p_owner)
        , _notified (false)
        { }

    void task_handle::notify()
    {
        _notified = true;
        _owner.task_notified(*this);
    }

    bool task_handle::notified() const
        { return _notified; }

    bool task_handle::poll()
    {
        _notified = false;
        return poll_impl();
    }

    /* task_handle_tpl */

    template<typename T_future>
    template<typename X_future>
    task_handle_tpl<T_future>
        ::task_handle_tpl(
            executor&   p_owner,
            X_future&&  p_future)
        : task_handle   (p_owner)
        , _future       (std::forward<X_future>(p_future))
        { }

    template<typename T_future>
    bool task_handle_tpl<T_future>
        ::poll_impl()
        { return _future.poll().is_ready(); }

} } }
