#pragma once

#include <cppfs/directory.h>

#include <asyncpp/core/misc/chaining.h>

namespace asyncpp {
namespace fs {

    struct directory
    {
    public:
        cppfs::directory handle;

    public:
        /**
         * @brief Constructor.
         */
        inline directory(const cppfs::path& p_path);

    public:
        /**
         * @brief Create a new future to create a directory.
         */
        template<chaining_mode X_mode = copy>
        inline auto create() &;

        /**
         * @brief Create a new future to create a directory.
         */
        template<chaining_mode X_mode = move>
        inline auto create() &&;

    public:
        /**
         * @brief Create a new future to create a directory and all it's parent directories.
         */
        template<chaining_mode X_mode = copy>
        inline auto create_all() &;

        /**
         * @brief Create a new future to create a directory and all it's parent directories.
         */
        template<chaining_mode X_mode = move>
        inline auto create_all() &&;

    public:
        /**
         * @brief Create future to read the content of a directory.
         */
        template<chaining_mode X_mode = copy>
        inline auto read() &;

        /**
         * @brief Create future to read the content of a directory.
         */
        template<chaining_mode X_mode = move>
        inline auto read() &&;

    public:
        /**
         * @brief Create future to read the content of a directory including the subdirectories.
         */
        template<chaining_mode X_mode = copy>
        inline auto read_all() &;

        /**
         * @brief Create future to read the content of a directory including the subdirectories.
         */
        template<chaining_mode X_mode = move>
        inline auto read_all() &&;

    public:
        /**
         * @brief Create a new future to remove an empty directory.
         */
        template<chaining_mode X_mode = copy>
        inline auto remove() &;

        /**
         * @brief Create a new future to remove an empty directory.
         */
        template<chaining_mode X_mode = move>
        inline auto remove() &&;

    public:
        /**
         * @brief Create a new future to remove a directory and all it's content.
         */
        template<chaining_mode X_mode = copy>
        inline auto remove_all() &;

        /**
         * @brief Create a new future to remove a directory and all it's content.
         */
        template<chaining_mode X_mode = move>
        inline auto remove_all() &&;
    };

} }
