#pragma once

#include <asyncpp/core/misc.h>
#include <asyncpp/core/future/future.h>

#include "delay.pre.h"
#include "../timer/timer.h"
#include "../timer/resource.h"

namespace asyncpp {
namespace timing {

    struct delay final
        : public base_future<void, delay>
    {
    public:
        using value_type        = void;
        using base_future_type  = base_future<void, delay>;
        using result_type       = typename base_future_type::result_type;

    private:
        time_point              _deadline;
        __impl::resource_ptr_w  _resource;

    public:
        /**
         * @brief Constructor. Create a delay at the given deadline.
         */
        inline delay(const time_point& p_deadline);

        /**
         * @brief Constructor. Create a delay with the given duration.
         */
        template<typename T_base, typename T_ratio>
        inline delay(const duration<T_base, T_ratio>& p_duration);

        /**
         * @brief Destructor.
         */
        inline ~delay();

        /**
         * @brief Get the current deadline of the delay.
         */
        inline time_point deadline() const;

        /**
         * @brief Reset the delay to a new time point.
         */
        inline void reset(const time_point& p_deadline);

        /**
         * @brief Reset the delay to the given duration.
         */
        template<typename T_base, typename T_ratio>
        inline void reset(const duration<T_base, T_ratio>& p_duration);

    public: /* future */
        /**
         * @brief Poll the result from the future.
         */
        inline result_type poll();
    };

} }
