#pragma once

#include <asyncpp/core/result.h>
#include <asyncpp/core/future/future.pre.h>

#include "delay.h"

#include "../timer/timer_base.inl"

namespace asyncpp {
namespace timing {

    /* delay */

    delay::delay(const time_point& p_deadline)
        : _deadline (p_deadline)
        , _resource ()
        { }

    template<typename T_base, typename T_ratio>
    delay::delay(const duration<T_base, T_ratio>& p_duration)
        : _deadline (asyncpp::now() + p_duration)
        , _resource ()
        { }

    inline delay::~delay()
        { __impl::timer_base::unregister_resource(_resource); }

    time_point delay::deadline() const
        { return _deadline; }

    void delay::reset(const time_point& p_deadline)
    {
        __impl::timer_base::unregister_resource(_resource);
        _deadline = p_deadline;
    }

    template<typename T_base, typename T_ratio>
    void delay::reset(const duration<T_base, T_ratio>& p_duration)
    {
        __impl::timer_base::unregister_resource(_resource);
        _deadline = asyncpp::now() + p_duration;
    }

    typename delay::result_type
    delay
        ::poll()
    {
        auto now = asyncpp::now();

        if (_deadline <= now)
            return result_type::ready();

        if (_resource.expired())
            _resource = timing::__impl::timer_base::register_resource(_deadline);

        return result_type::not_ready();
    }

} }
