#pragma once

#include <asyncpp/core/misc.h>
#include <asyncpp/core/future/future.h>
#include <asyncpp/core/stream/stream.h>

#include "../delay/delay.h"

namespace asyncpp {
namespace timing {

    struct interval final
        : public base_stream<void, interval>
    {
    public:
        using value_type        = void;
        using base_stream_type  = base_stream<void, delay>;
        using result_type       = typename base_stream_type::result_type;

    private:
        delay               _delay;     //!< Delay future
        clock::duration     _duration;  //!< Interval duration.
        time_point          _deadline;  //!< Deadline after the interval should end.

    public:
        /**
         * @brief Constructor.
         */
        template<typename T_base, typename T_ratio>
        inline interval(
            const duration<T_base, T_ratio>&    p_duration,
            time_point                          p_deadline = time_point());

        /**
         * @brief Constructor.
         */
        template<typename T_base, typename T_ratio>
        inline interval(
            const time_point&                   p_at,
            const duration<T_base, T_ratio>&    p_duration,
            time_point                          p_deadline = time_point());

        /**
         * @brief Get the duration of the interval.
         */
        inline const clock::duration& duration() const;

    public: /* stream */
        /**
         * @brief Poll the result from the stream.
         */
        inline result_type poll();
    };

} }
