#pragma once

#include "interval.h"

#include "../delay/delay.inl"

namespace asyncpp {
namespace timing {

    /* interval */

    template<typename T_base, typename T_ratio>
    interval::interval(
        const ::asyncpp::duration<T_base, T_ratio>& p_duration,
        time_point                                  p_deadline)
            : interval(asyncpp::now() + p_duration, p_duration, p_deadline)
            { }

    template<typename T_base, typename T_ratio>
    interval::interval(
        const time_point&                           p_at,
        const ::asyncpp::duration<T_base, T_ratio>& p_duration,
        time_point                                  p_deadline)
            : _delay   (p_at)
            , _duration(p_duration)
            , _deadline(p_deadline)
            { }

    const clock::duration& interval::duration() const
        { return _duration; }

    typename interval::result_type
    interval
        ::poll()
    {
        if (    _deadline.time_since_epoch().count()
            &&  _delay.deadline() >= _deadline
            &&  asyncpp::now()    >= _delay.deadline())
            return result_type::done();

        auto ret = _delay.poll();
        if (ret.is_not_ready())
            return result_type::not_ready();

        auto now = _delay.deadline();
        auto new_deadline = now + _duration;
        _delay.reset(new_deadline);

        return result_type::ready();
    }

} }
