#pragma once

#include "timeout_future.h"

namespace asyncpp {
namespace timing {

    /* timeout_future */

    template<
        typename T_inner>
    template<
        typename X_inner,
        typename X_base,
        typename X_ratio>
    timeout_future<T_inner>::timeout_future(
        X_inner&&                           p_inner,
        const duration<X_base, X_ratio>&    p_timeout)
        : _inner    (std::forward<X_inner>(p_inner))
        , _timeout  (p_timeout)
        , _delay    (asyncpp::now() + p_timeout)
        { }

    template<
        typename T_inner>
    template<
        typename X_base,
        typename X_ratio>
    void timeout_future<T_inner>
        ::reset(const duration<X_base, X_ratio>& p_timeout)
    {
        _timeout = p_timeout;
        _delay.reset(_timeout);
    }

    template<
        typename T_inner>
    typename timeout_future<T_inner>::result_type
    timeout_future<T_inner>
        ::poll()
    {
        auto r = _inner.poll();

        if (    r.is_not_ready()
            &&  _delay.poll())
        {
            auto new_deadline = _delay.deadline() + _timeout;
            _delay.reset(new_deadline);
            throw timing::timeout_exception();
        }

        return r;
    }

} }
