#pragma once

#include <asyncpp/core/stream/stream.h>

#include "../delay/delay.h"

namespace asyncpp {
namespace timing {

    template<typename T_inner>
    struct timeout_stream final
        : public base_stream<
            typename std::decay_t<T_inner>::value_type,
            timeout_stream<T_inner>>
    {
    public:
        using inner_type        = T_inner;
        using value_type        = typename std::decay_t<inner_type>::value_type;
        using this_type         = timeout_stream<inner_type>;
        using base_stream_type  = base_stream<value_type, this_type>;
        using result_type       = typename base_stream_type::result_type;

    private:
        inner_type      _inner;     //!< Inner stream.
        clock::duration _timeout;   //!< Timeout.
        delay           _delay;     //!< Delay stream.

    public:
        /**
         * @brief Constructor.
         */
        template<typename X_inner, typename X_base, typename X_ratio>
        inline timeout_stream(
            X_inner&&                           p_inner,
            const duration<X_base, X_ratio>&    p_timeout);

        /**
         * @brief Reset the deadline.
         */
        template<typename X_base, typename X_ratio>
        inline void reset(
            const duration<X_base, X_ratio>& p_timeout);

        /**
         * @brief Poll the result from the future.
         */
        inline result_type poll();
    };

} }
