#pragma once

#include <set>
#include <memory>

#include <cppcore/synchronization/locked.h>

#include "timer_impl.h"

namespace asyncpp {
namespace timing {

    template<typename T_inner = void>
    struct timer
        : public __impl::timer_impl<T_inner>
    {
    public:
        using base_type  = __impl::timer_impl<T_inner>;
        using inner_type = T_inner;

    public:
        /**
         * @brief Constructor.
         */
        template<typename... X_args>
        inline timer(X_args&&... p_args);

        /**
         * @brief Handle idle of the runtime.
         *
         * This method is called as soon as the runtime has nothing to do.
         * The passed deadline is the timepoint the method should return (or null if not set).
         */
        inline void idle(const time_point * p_deadline);

        /**
         * @brief This method is calld by the executor when a new thread needs to be initialized.
         *
         * @return Object to hold as long as the thread is active on this timer.
         */
        inline auto init_thread();
    };

} }
