#pragma once

#include <set>
#include <memory>

#include <cppcore/synchronization/locked.h>

#include <asyncpp/core/misc.h>

#include "timer_base.pre.h"
#include "resource.h"

namespace asyncpp {
namespace timing {
namespace __impl {

    struct timer_base
    {
    private:
        struct resource_less_compare
        {
            constexpr bool operator()(
                const resource_ptr_s& lhs,
                const resource_ptr_s& rhs) const;
        };

        using resource_set = std::set<resource_ptr_s, resource_less_compare>;

    protected:
        cppcore::locked<resource_set> _resources;

    public:
        /**
         * @brief Get the number of registered timing resources.
         */
        inline size_t resource_count();

    public:
        /**
         * @brief Get the thread local timer_base instance.
         */
        static inline timer_base* current();

        /**
         * @brief Register a new resource within this timer_base.
         */
        static inline resource_ptr_w register_resource(const time_point& p_deadline);

        /**
         * @brief Register a new resource within this timer_base.
         */
        static inline void unregister_resource(resource_ptr_w& p_value);

    private:
        /**
         * @brief Create resource for a new resource.
         */
        resource_ptr_w create_resource(const time_point& p_deadline);

    public:
        struct lock
        {
        public:
            /**
             * @brief Constructor.
             */
            inline lock(timer_base& p_timer);

            /**
             * @brief Destructor.
             */
            inline ~lock();
        };

    private:
        struct storage
        {
            timer_base* current { nullptr };
        };

        /**
         * @brief Get the thread local storage.
         */
        static inline storage& local_storage();
    };

} } }
