#pragma once

#include <cppcore/misc/exception.h>

#include "timer_base.h"

namespace asyncpp {
namespace timing {
namespace __impl {

    /* timer_base::inner_less_compare */

    constexpr bool timer_base::resource_less_compare::operator()(
        const resource_ptr_s& lhs,
        const resource_ptr_s& rhs) const
    {
        return  (lhs->deadline <  rhs->deadline)
                    ?  true
                    :
                (lhs->deadline == rhs->deadline)
            &&  (lhs.get()     <  rhs.get())
                    ?   true
                    :   false;
    }

    /* timer_base::lock */

    timer_base::lock::lock(timer_base& p_timer)
    {
        auto& current = timer_base::local_storage().current;
        if (current)
            throw cppcore::invalid_operation_exception("Current timer is already assigned!");
        current = &p_timer;
    }

    timer_base::lock::~lock()
        { timer_base::local_storage().current = nullptr; }

    /* timer_base */

    size_t timer_base::resource_count()
        { return _resources.lock()->size(); }

    timer_base* timer_base::current()
        { return local_storage().current; }

    resource_ptr_w timer_base::register_resource(const time_point& p_deadline)
    {
        auto t = current();

        if (!t)
            throw cppcore::invalid_operation_exception("Thread local timer_base instance is not assigned!");

        return t->create_resource(p_deadline);
    }

    void timer_base::unregister_resource(resource_ptr_w& p_value)
    {
        auto s = p_value.lock();
        p_value.reset();
        if (s)
            s->owner._resources.lock()->erase(s);
    }

    resource_ptr_w timer_base::create_resource(const time_point& p_deadline)
    {
        auto r = std::make_shared<resource>(
            *this,
            p_deadline,
            task::current());

        _resources.lock()->insert(r);

        return r;
    }

    timer_base::storage& timer_base::local_storage()
    {
        thread_local storage value;
        return value;
    }

} } }
