#pragma once

#include "timer_impl.h"
#include "timer_base.h"

namespace asyncpp {
namespace timing {
namespace __impl {

    /* timer_impl - default */

    template<typename T_inner>
    struct timer_impl
        : public timer_base
    {
    public:
        using inner_type                = T_inner;
        using inner_thread_lock_type    = decltype(std::declval<inner_type>().init_thread());

        struct thread_lock
        {
            timer_base::lock        timer_lock;
            inner_thread_lock_type  inner_lock;

            template<typename... X_args>
            inline thread_lock(timer_base& p_timer, X_args&&... p_args)
                : timer_lock(p_timer)
                , inner_lock(std::forward<X_args>(p_args)...)
                { }

            inline thread_lock(thread_lock &&) = delete;
            inline thread_lock(thread_lock const &) = delete;
        };

    private:
        inner_type  _inner;

    public:
        template<typename X_inner>
        inline timer_impl(X_inner&& p_inner)
            : _inner(std::forward<X_inner>(p_inner))
            { }

    protected:
        inline void idle_impl(const time_point * p_deadline)
        {
            {
            auto r = _resources.lock();
            if (!r->empty())
            {
                p_deadline = merge_deadlines(p_deadline, &(*r->begin())->deadline);
            }
            }

            _inner.idle(p_deadline);
        }

        inline auto init_thread_impl()
            { return thread_lock(*this, _inner.init_thread()); }
    };

    /* timer_impl - void */

    template<>
    struct timer_impl<void>
        : public timer_base
    {
    protected:
        inline void idle_impl(const time_point * p_deadline)
            { }

        inline auto init_thread_impl()
            { return timer_base::lock(*this); }
    };

} } }
