#include <gtest/gtest.h>

#include <asyncpp.h>

#include "../../helper/test_delay.h"

using namespace ::testing;
using namespace ::asyncpp;

TEST(future_tests, poll)
{
    test_delay f(5, 0);

    auto r0 = f.poll();
    ASSERT_FALSE(r0);

    auto r1 = f.poll();
    ASSERT_FALSE(r1);

    auto r2 = f.poll();
    ASSERT_FALSE(r2);

    auto r3 = f.poll();
    ASSERT_FALSE(r3);

    auto r4 = f.poll();
    ASSERT_FALSE(r4);

    auto r = f.poll();
    ASSERT_TRUE (r);
    ASSERT_EQ   (5,   *r);
    ASSERT_EQ   (5,    f._count);
    ASSERT_EQ   (&*r, &f._count);
}

TEST(future_tests, map)
{
    test_delay d(1, 1);
    auto f = d
        .map([](int& i){
            return 10 + i;
        });

    auto r = f.poll();
    ASSERT_TRUE(r);
    ASSERT_EQ  (11, *r);
}

TEST(future_tests, and_then)
{
    test_delay d(2, 0);
    auto f = d
        .and_then([](int& i){
            return test_delay { 5, i };
        });

    auto r0 = f.poll();
    ASSERT_FALSE(r0);

    auto r1 = f.poll();
    ASSERT_FALSE(r1);

    auto r2 = f.poll();
    ASSERT_FALSE(r2);

    auto r3 = f.poll();
    ASSERT_FALSE(r3);

    auto r4 = f.poll();
    ASSERT_FALSE(r4);

    auto r = f.poll();
    ASSERT_TRUE(r);
    ASSERT_EQ  (5, *r);
}

TEST(future_tests, lazy)
{
    int i = 1;
    auto f = lazy([&]{
        ++i;
        return i + 1;
    });

    ASSERT_EQ(i, 1);

    auto r = f.poll();

    ASSERT_TRUE(r);
    ASSERT_EQ  (2, i);
    ASSERT_EQ  (3, *r);
}
