#include <gtest/gtest.h>

#include "../../helper/now_mock.h"

#include <asyncpp.h>

using namespace ::testing;
using namespace ::asyncpp;
using namespace ::asyncpp::timing;

TEST(interval_tests, poll)
{
    InSequence seq;
    StrictMock<now_mock> m;
    asyncpp::timing::timer t;
    interval s(
        time_point(std::chrono::seconds(10)),
        std::chrono::seconds(5),
        time_point(std::chrono::seconds(30)));

    // 30 - 10 = 20
    // 20 / 5  = 4
    // 4 x Ready

    timing::__impl::timer_base::lock l(t);

    EXPECT_CALL(m, now)
        .WillOnce(Return(time_point(std::chrono::seconds(0))));

    auto r0 = s.poll();
    ASSERT_EQ(result_status::not_ready, r0.status());

    EXPECT_CALL(m, now)
        .WillOnce(Return(time_point(std::chrono::seconds(9))));

    auto r1 = s.poll();
    ASSERT_EQ(result_status::not_ready, r1.status());

    EXPECT_CALL(m, now)
        .WillOnce(Return(time_point(std::chrono::seconds(10))));

    auto r2 = s.poll(); // ready @ 10sec
    ASSERT_EQ(result_status::ready, r2.status());

    EXPECT_CALL(m, now)
        .WillOnce(Return(time_point(std::chrono::seconds(11))));

    auto r3 = s.poll();
    ASSERT_EQ(result_status::not_ready, r3.status());

    EXPECT_CALL(m, now)
        .WillOnce(Return(time_point(std::chrono::seconds(30))));

    auto r4 = s.poll(); // ready @ 15sec
    ASSERT_EQ(result_status::ready, r4.status());

    EXPECT_CALL(m, now)
        .WillOnce(Return(time_point(std::chrono::seconds(30))));

    auto r5 = s.poll(); // ready @ 20sec
    ASSERT_EQ(result_status::ready, r5.status());

    EXPECT_CALL(m, now)
        .WillOnce(Return(time_point(std::chrono::seconds(30))));

    auto r6 = s.poll(); // ready @ 25sec
    ASSERT_EQ(result_status::ready, r6.status());

    EXPECT_CALL(m, now)
        .WillOnce(Return(time_point(std::chrono::seconds(30))));

    auto r7 = s.poll();
    ASSERT_EQ(result_status::done, r7.status());
}
